# this script has been tested on Windows 10 using the mdf2parquet.exe v2.3.2
# update the script with your container names and connection strings to run it
# and make sure to place the MF4 decoder next to the script

# WARNING: The script is deliberately light and includes no error handling.
# Use it at your own risk. Also, consider a step-by-step approach for large data volumes

from azure.storage.blob import BlobServiceClient, BlobClient
import os, sys, subprocess


# define variables
container_input = "input-container-name"
container_output = "output-container-name"
connection_string_input = "your-input-connection-string"
connection_string_output = "your-output-connection-string"

decoder_path = "mdf2parquet_decode.exe"
fs_logfile_path = "input"
fs_parquet_path = "output"

# initialize
blob_service_client_input = BlobServiceClient.from_connection_string(
    connection_string_input
)
container_client_input = blob_service_client_input.get_container_client(container_input)

blob_service_client_output = BlobServiceClient.from_connection_string(
    connection_string_output
)
container_client_output = blob_service_client_input.get_container_client(
    container_output
)

os.makedirs(fs_logfile_path, exist_ok=True)
os.makedirs(fs_parquet_path, exist_ok=True)

# List the folders (device IDs) and files in each folder
blob_list = container_client_input.walk_blobs(name_starts_with="")
deviceids = set(blob.name.split("/")[0] for blob in blob_list if "/" in blob.name)

# Valid file extensions
valid_extensions = {".MF4", ".MFC", ".MFE", ".MFM"}
downloaded_files = 0

for deviceid in deviceids:
    # Create directory for device ID
    device_dir = os.path.join(fs_logfile_path, deviceid)
    os.makedirs(device_dir, exist_ok=True)

    # Handle session folders under each device ID
    session_folders = set(
        blob.name.split("/")[1]
        for blob in container_client_input.walk_blobs(name_starts_with=f"{deviceid}/")
        if len(blob.name.split("/")) > 1
    )

    for session in session_folders:
        session_dir = os.path.join(device_dir, session)
        os.makedirs(session_dir, exist_ok=True)

        # Filter blobs by device ID, session folder, and check extension
        for blob in container_client_input.walk_blobs(
            name_starts_with=f"{deviceid}/{session}/"
        ):
            if os.path.splitext(blob.name)[1].upper() in valid_extensions:
                blob_client = container_client_input.get_blob_client(blob)
                download_path = os.path.join(session_dir, os.path.basename(blob.name))
                with open(download_path, "wb") as download_file:
                    download_file.write(blob_client.download_blob().readall())
                    downloaded_files += 1

print(f"Completed downloading {downloaded_files} MDF files")

# -----------
# process downloaded files

result = subprocess.run(
    [
        str(decoder_path),
        "-i",
        str(fs_logfile_path),
        "-O",
        str(fs_parquet_path),
        "--verbosity=5",
        "-X",
    ],
    text=True,
    capture_output=True,
)

print(f"\nProcessed downloaded MDF log files via the mdf2parquet decoder:")
print(f"{result.stdout}")
print(f"{result.stderr}")


# -----------
# upload resulting Parquet files
uploaded_files = 0
for root, dirs, files in os.walk(fs_parquet_path):
    for file in files:
        if file.endswith(".parquet"):
            file_path = os.path.join(root, file)
            blob_path = os.path.relpath(file_path, fs_parquet_path)
            blob_client = container_client_output.get_blob_client(blob=blob_path)
            with open(file_path, "rb") as data:
                blob_client.upload_blob(data, overwrite=True)
                uploaded_files += 1

print(f"Completed uploading {uploaded_files} Parquet files to {container_output}.")
